﻿// Copyright (C) 2011 Oliver Sturm <oliver@oliversturm.com>. All rights reserved.

using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Linq.Expressions;

namespace CompilerGenerated {
  class Program {
    static void Main(string[] args) {
      // Here's a simple lambda expression:
      Func<int, int> square = x => x * x;

      // By changing the type of the variable it gets stored in, you can 
      // have the compiler generate an expression tree instead:
      Expression<Func<int, int>> squareExp = x => x * x;

      // You can execute the lambda expression directly:
      Console.WriteLine(square(25));

      // The expression tree can't be executed directly, but it can be
      // analyzed, printed out, modified, ....:
      ExpressionDumper.Output(squareExp);

      // You can also compile the tree into a lambda expression that can
      // then be executed:
      var square_ = squareExp.Compile( );
      Console.WriteLine(square_(25));

      // Calling DoSomething and DoSomethingElse has the 
      // same call semantics:
      DoSomething(x => x * x);
      DoSomethingElse(x => x * x);

      // Btw - (obviously?!?) I can't do this instead:
      // DoSomethingElse(square);
    }

    static void DoSomething(Func<int, int> modifyValue) {
      // ... now I can call the delegate
    }

    static void DoSomethingElse(Expression<Func<int, int>> modifyValueExp) {
      // ... now I can compile the delegate and call it, or 
      // I can decide to analyze, extend or change it before using it.
    }
  }
}
