﻿// Copyright (C) 2011 Oliver Sturm <oliver@oliversturm.com>. All rights reserved.

using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Linq.Expressions;

namespace Fact {
  class Program {
    static void Main(string[] args) {
      // The function Fact is a simple function to calculate a factorial
      Console.WriteLine(Fact(0)); // 1
      Console.WriteLine(Fact(1)); // 1
      Console.WriteLine(Fact(5)); // 120

      // it's also possible to do the same thing in a lambda
      Func<int, int> fact = x => {
        int result = 1;
        for (int m = 2; m <= x; m++)
          result *= m;
        return result;
      };
      // same results
      Console.WriteLine(fact(0)); // 1
      Console.WriteLine(fact(1)); // 1
      Console.WriteLine(fact(5)); // 120

      // this doesn't compile - no support for statement bodies in
      // expression trees, as far as C# is concerned
      //Expression<Func<int, int>> factExpression = x => {
      //  int result = 1;
      //  for (int m = 2; m <= x; m++)
      //    result *= m;
      //  return result;
      //};


      // now get hold of a version of fact constructed as an 
      // expression tree
      var factRuntime = ConstructRuntimeFact( );
      // same results again
      Console.WriteLine(factRuntime(0)); // 1
      Console.WriteLine(factRuntime(1)); // 1
      Console.WriteLine(factRuntime(5)); // 120
    }

    static int Fact(int x) {
      int result = 1;
      for (int m = 2; m <= x; m++)
        result *= m;
      return result;
    }

    static Func<int, int> ConstructRuntimeFact( ) {
      var param = Expression.Parameter(typeof(int), "x");
      var resultVar = Expression.Variable(typeof(int), "result");
      var mVar = Expression.Variable(typeof(int), "m");
      var loopEnd = Expression.Label( );

      return Expression.Lambda<Func<int, int>>(
        Expression.Block(
          new[] { resultVar },
          Expression.Assign(resultVar, Expression.Constant(1)),
          Expression.Block(
            new[] { mVar },
            Expression.Assign(mVar, Expression.Constant(2)),
            Expression.Loop(
              Expression.Block(
                Expression.IfThen(
                  Expression.Not(
                    Expression.LessThanOrEqual(mVar, param)),
                  Expression.Break(loopEnd)),
                Expression.MultiplyAssign(resultVar, mVar),
                Expression.PostIncrementAssign(mVar)),
              loopEnd)),
          resultVar),
        param).Compile( );
    }
  }
}
